import numpy as np
import pandas as pd 
from ampligraph.evaluation import train_test_split_no_unseen
from ampligraph.datasets import load_from_csv
from ampligraph.latent_features import TransE
from ampligraph.evaluation import select_best_model_ranking, evaluate_performance, mrr_score, hits_at_n_score
import tensorflow as tf

#loading the datasets and creating train test val sets
X = load_from_csv("../data/BengIwnData", "graph.csv", sep = ',')
X_train, X_test = train_test_split_no_unseen(X, test_size = 0.2)
X_train, X_valid = train_test_split_no_unseen(X_train, test_size = 0.2)
print(f"The shape of training set: {X_train.shape}")
print(f"The shape of test set: {X_test.shape}")
print(f"The shape of validation set: {X_valid}")

#tuning hyperparams 
model_class = TransE
#param_grid = {
#        "k": [50, 100],
#        "epochs": 50,
#        "eta": [5, 10, 15],
#        "embedding_model_params": {
#            },
#        "optimizer": ['adam', 'adagrad'],
#        "regularizer": ["LP", None],
#        "verbose": True
#        }
#best_model, best_params, _, _, _, _ = select_best_model_ranking(model_class, X_train, X_valid, X_test, param_grid, verbose = True, early_stopping = True)
#print("The best Params obtained are:", best_params)


# fit
# The best Params obtained are: {'k': 50, 'epochs': 50, 'eta': 5, 'optimizer': 'adam', 'regularizer': None, 'verbose': True}

model = TransE(eta = 5, seed = 0, k = 50, epochs =40, optimizer = 'adam', regularizer = None)
model.fit(X_train, early_stopping = True, early_stopping_params={'x_valid': X_valid, 'criteria': 'mrr'})

test_ranks = evaluate_performance(X_test, model)
val_ranks = evaluate_performance(X_valid, model)

print("Validation results:")
print("MRR =", mrr_score(val_ranks))
print("Hits at 10 score =", hits_at_n_score(val_ranks, n=10))
print("Test results:")
print("MRR =", mrr_score(test_ranks))
print("Hits at 10 score =", hits_at_n_score(test_ranks, n=10))
